'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GraphQLConfig = exports.GraphQLRC = undefined;
exports.findGraphQLConfigDir = findGraphQLConfigDir;
exports.getGraphQLConfig = getGraphQLConfig;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } } /**
                                                                                                                                                           *  Copyright (c) Facebook, Inc.
                                                                                                                                                           *  All rights reserved.
                                                                                                                                                           *
                                                                                                                                                           *  This source code is licensed under the license found in the
                                                                                                                                                           *  LICENSE file in the root directory of this source tree.
                                                                                                                                                           *
                                                                                                                                                           *  
                                                                                                                                                           */

var CONFIG_LIST_NAME = 'build-configs';
var SCHEMA_PATH = 'schema-file';
var CUSTOM_VALIDATION_RULES_MODULE_PATH = 'custom-validation-rules';

/**
 * Finds a .graphqlrc configuration file, and returns null if not found.
 * If the file isn't present in the provided directory path, walk up the
 * directory tree until the file is found or it reaches the root directory.
 */
function findGraphQLConfigDir(dirPath) {
  var currentPath, filePath;
  return regeneratorRuntime.async(function findGraphQLConfigDir$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          currentPath = _path2.default.resolve(dirPath);
          filePath = void 0;

        case 2:
          if (!(currentPath.length > 1)) {
            _context.next = 9;
            break;
          }

          filePath = _path2.default.join(currentPath, '.graphqlrc');

          if (!_fs2.default.existsSync(filePath)) {
            _context.next = 6;
            break;
          }

          return _context.abrupt('break', 9);

        case 6:

          currentPath = _path2.default.dirname(currentPath);
          _context.next = 2;
          break;

        case 9:
          return _context.abrupt('return', filePath ? currentPath : null);

        case 10:
        case 'end':
          return _context.stop();
      }
    }
  }, null, this);
}

function getGraphQLConfig(configDir) {
  var rawGraphQLConfig, graphqlrc;
  return regeneratorRuntime.async(function getGraphQLConfig$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          _context2.next = 2;
          return regeneratorRuntime.awrap(new Promise(function (resolve, reject) {
            return _fs2.default.readFile(_path2.default.join(configDir, '.graphqlrc'), 'utf8', function (error, response) {
              if (error) {
                // eslint-disable-next-line no-console
                console.error('.graphqlrc file is not available in the provided config ' + ('directory: ' + configDir + '\nPlease check the config directory ') + 'path and try again.');
                reject();
              }
              resolve(response);
            });
          }));

        case 2:
          rawGraphQLConfig = _context2.sent;
          _context2.prev = 3;
          graphqlrc = JSON.parse(rawGraphQLConfig);
          return _context2.abrupt('return', new GraphQLRC(graphqlrc, configDir));

        case 8:
          _context2.prev = 8;
          _context2.t0 = _context2['catch'](3);

          // eslint-disable-next-line no-console
          console.error('Parsing JSON in .graphqlrc file has failed.');
          throw new Error(_context2.t0);

        case 12:
        case 'end':
          return _context2.stop();
      }
    }
  }, null, this, [[3, 8]]);
}

var GraphQLRC = exports.GraphQLRC = function () {
  function GraphQLRC(graphqlrc, root) {
    var _this = this;

    _classCallCheck(this, GraphQLRC);

    this._graphqlrc = graphqlrc;
    this._rootDir = root;
    this._configs = {};
    if (this._graphqlrc[CONFIG_LIST_NAME]) {
      Object.keys(this._graphqlrc[CONFIG_LIST_NAME]).forEach(function (name) {
        _this._configs[name] = new GraphQLConfig(name, _this._graphqlrc[CONFIG_LIST_NAME][name], _this._rootDir);
      });
    }
  }

  GraphQLRC.prototype.getConfigDir = function getConfigDir() {
    return this._rootDir;
  };

  GraphQLRC.prototype.getConfigNames = function getConfigNames() {
    return Object.keys(this._graphqlrc[CONFIG_LIST_NAME]);
  };

  GraphQLRC.prototype.getConfig = function getConfig(name) {
    var config = this._configs[name];
    if (config === undefined) {
      throw new Error('Config ' + name + ' not defined. Choose one of: ' + Object.keys(this._graphqlrc[CONFIG_LIST_NAME]).join(', '));
    }
    return config;
  };

  GraphQLRC.prototype.getConfigByFilePath = function getConfigByFilePath(filePath) {
    var _this2 = this;

    var name = this.getConfigNames().find(function (configName) {
      return _this2.getConfig(configName).isFileInInputDirs(filePath);
    });

    return name ? this._configs[name] : null;
  };

  return GraphQLRC;
}();

var GraphQLConfig = exports.GraphQLConfig = function () {
  function GraphQLConfig(name, config, rootDir) {
    _classCallCheck(this, GraphQLConfig);

    this._name = name;
    this._rootDir = rootDir;
    this._config = config;
  }

  GraphQLConfig.prototype.getRootDir = function getRootDir() {
    return this._rootDir;
  };

  GraphQLConfig.prototype.getName = function getName() {
    return this._name;
  };

  GraphQLConfig.prototype.getConfig = function getConfig() {
    return this._config;
  };

  GraphQLConfig.prototype.getInputDirs = function getInputDirs() {
    return this._config['input-dirs'] ? this._config['input-dirs'] : [];
  };

  GraphQLConfig.prototype.getExcludeDirs = function getExcludeDirs() {
    return this._config['exclude-dirs'] ? this._config['exclude-dirs'] : [];
  };

  GraphQLConfig.prototype.isFileInInputDirs = function isFileInInputDirs(fileName) {
    if (!this.getInputDirs()) {
      return false;
    }
    return this.getInputDirs().some(function (dirPath) {
      return fileName.indexOf(dirPath) !== -1;
    });
  };

  GraphQLConfig.prototype.getSchemaPath = function getSchemaPath() {
    return this._config[SCHEMA_PATH] || null;
  };

  GraphQLConfig.prototype.getCustomValidationRulesModulePath = function getCustomValidationRulesModulePath() {
    var modulePath = this._config[CUSTOM_VALIDATION_RULES_MODULE_PATH];
    if (!modulePath) {
      return null;
    }
    return this._normalizePath(modulePath);
  };

  GraphQLConfig.prototype._normalizePath = function _normalizePath(modulePath) {
    var resolvedPath = void 0;
    if (modulePath.startsWith('~')) {
      // home directory
      var homeDirPath = process.platform === 'win32' ? process.env.USERPROFILE : process.env.HOME;
      resolvedPath = _path2.default.join(homeDirPath || '', modulePath.slice(1));
    } else if (modulePath.startsWith('./')) {
      // relative local directory
      resolvedPath = _path2.default.join(this._rootDir, modulePath);
    } else {
      // `/` or an actual module name (node_modules)
      resolvedPath = modulePath;
    }

    return resolvedPath;
  };

  return GraphQLConfig;
}();