'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getDiagnostics = getDiagnostics;

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _graphql = require('graphql');

var _CharacterStream = require('../parser/CharacterStream');

var _CharacterStream2 = _interopRequireDefault(_CharacterStream);

var _onlineParser = require('../parser/onlineParser');

var _onlineParser2 = _interopRequireDefault(_onlineParser);

var _Range = require('../utils/Range');

var _validateWithCustomRules = require('../utils/validateWithCustomRules');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var SEVERITY = {
  ERROR: 1,
  WARNING: 2,
  INFORMATION: 3,
  HINT: 4
}; /**
    *  Copyright (c) Facebook, Inc.
    *  All rights reserved.
    *
    *  This source code is licensed under the license found in the
    *  LICENSE file in the root directory of this source tree.
    *
    *  
    */

function getDiagnostics(queryText) {
  var schema = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
  var customRules = arguments[2];

  var ast = null;
  try {
    ast = (0, _graphql.parse)(queryText);
  } catch (error) {
    var range = getRange(error.locations[0], queryText);

    return [{
      severity: SEVERITY.ERROR,
      message: error.message,
      source: 'GraphQL: Syntax',
      range: range
    }];
  }

  var errors = schema ? (0, _validateWithCustomRules.validateWithCustomRules)(schema, ast, customRules) : [];
  return mapCat(errors, function (error) {
    return errorAnnotations(error);
  });
}

// General utility for map-cating (aka flat-mapping).
function mapCat(array, mapper) {
  return Array.prototype.concat.apply([], array.map(mapper));
}

function errorAnnotations(error) {
  if (!error.nodes) {
    return [];
  }
  return error.nodes.map(function (node) {
    var highlightNode = node.kind !== 'Variable' && node.name ? node.name : node.variable ? node.variable : node;

    (0, _assert2.default)(error.locations, 'GraphQL validation error requires locations.');
    var loc = error.locations[0];
    var end = loc.column + (highlightNode.loc.end - highlightNode.loc.start);
    return {
      source: 'GraphQL: Validation',
      message: error.message,
      severity: SEVERITY.ERROR,
      range: new _Range.Range(new _Range.Position(loc.line - 1, loc.column - 1), new _Range.Position(loc.line - 1, end))
    };
  });
}

function getRange(location, queryText) {
  var parser = (0, _onlineParser2.default)();
  var state = parser.startState();
  var lines = queryText.split('\n');

  (0, _assert2.default)(lines.length >= location.line, 'Query text must have more lines than where the error happened');

  var stream = null;

  for (var i = 0; i < location.line; i++) {
    stream = new _CharacterStream2.default(lines[i]);
    while (!stream.eol()) {
      var style = parser.token(stream, state);
      if (style === 'invalidchar') {
        break;
      }
    }
  }

  (0, _assert2.default)(stream, 'Expected Parser stream to be available.');

  var line = location.line - 1;
  var start = stream.getStartOfToken();
  var end = stream.getCurrentPosition();

  return new _Range.Range(new _Range.Position(line, start), new _Range.Position(line, end));
}